from __future__ import annotations

CONFIG_BASENAME = "config"
CONFIG_ENVVAR = "BIRRE_CONFIG"
DEFAULT_CONFIG_FILENAME = f"{CONFIG_BASENAME}.toml"
LOCAL_CONFIG_FILENAME = f"{CONFIG_BASENAME}.local.toml"

__all__ = [
    "CONFIG_BASENAME",
    "CONFIG_ENVVAR",
    "DEFAULT_CONFIG_FILENAME",
    "LOCAL_CONFIG_FILENAME",
]
